@@ -6,8 +6,9 @@
 #' @param digits number of digits to keep for the percentages
 #' @param cum if TRUE, display cumulative percentages
 #' @param total if TRUE, add a final row with totals
-#' @param exclude vector of values to exclude from the tabulation
+#' @param exclude vector of values to exclude from the tabulation (if \code{x} is a vector)
 #' @param sort if specified, allow to sort the table by increasing ("inc") or decreasing ("dec") frequencies
+#' @param valid if TRUE, display valid percentages
 #' @return
 #' The result is an object of class data.frame.
 #' @seealso
@@ -15,24 +16,47 @@
 #' @export
 
 `freq` <-
-function (x, digits=1, cum=FALSE, total=FALSE, exclude=NULL, sort="") {
-  if (is.factor(x)) x <- factor(x, exclude=exclude)
+function (x, digits=1, cum=FALSE, total=FALSE, exclude=NULL, sort="", valid=!(NA%in%exclude)) {
   if (is.table(x)) tab <- x
-  else tab <- table(x, exclude=exclude)
+  else tab <- table(x, exclude=exclude, useNA="ifany")
   effectifs <- as.vector(tab)
   pourc <- as.vector(effectifs/sum(effectifs)*100)
   result <- data.frame(n=effectifs, pourc=pourc)
   rownames(result) <- ifelse(is.na(names(tab)),"NA",names(tab))
+  if (valid) {
+    if (NA %in% names(tab)) {
+      NA.position <- which(is.na(names(tab)))
+      n.na <- tab[NA.position]
+      valid.pourc <- as.vector(effectifs/(sum(effectifs)-n.na)*100)
+      valid.pourc[NA.position] <- 0 # temporary 0 for cumsum
+    } else
+      valid.pourc <- pourc
+    result <- cbind(result, valid.pourc)
+  }
   if (sort=="inc") result <- result[order(result$n),]
   if (sort=="dec") result <- result[order(result$n, decreasing=TRUE),]
   if (total) result <- rbind(result, Total=apply(result,2,sum))
+  if (total & valid) 
+    result[length(result$pourc),"valid.pourc"] <- 100
   if (cum) {
     pourc.cum <- cumsum(result$pourc)
     if (total) pourc.cum[length(pourc.cum)] <- 100
     result <- cbind(result, pourc.cum)
+    if (valid) {
+      valid.pourc.cum <- cumsum(result$valid.pourc)
+      if (total) valid.pourc.cum[length(valid.pourc.cum)] <- 100
+      result <- cbind(result, valid.pourc.cum)
+    }
+  }
+  if (valid & NA %in% names(tab)) {
+    result["NA","valid.pourc"] <- NA
+    if (cum)
+      result["NA","valid.pourc.cum"] <- NA
   }
   names(result)[which(names(result)=="pourc")] <- "%"
+  names(result)[which(names(result)=="valid.pourc")] <- "val%"
   names(result)[which(names(result)=="pourc.cum")] <- "%cum"
+  names(result)[which(names(result)=="valid.pourc.cum")] <- "val%cum"
   round(result, digits=digits)
 }
 
@@ -100,6 +124,7 @@ freq.na <- function(data, ...) {
 #' @param total if \code{TRUE}, add a row with the sum of percentages and a column with global percentages
 #' @param percent if \code{TRUE}, add a percent sign after the values when printing
 #' @param drop if \code{TRUE}, lines or columns with a sum of zero, which would generate \code{NaN} percentages, are dropped.
+#' @param n if \code{TRUE}, display number of observations per column.
 #' @return
 #' The result is an object of class \code{table} and \code{proptab}.
 #' @seealso
@@ -115,18 +140,30 @@ freq.na <- function(data, ...) {
 #' @export
 
 `cprop` <-
-function (tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE) {
+function (tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE, n=FALSE) {
   # subset to non-empty rows/columns
-  if(drop) tab <- tab[rowSums(tab) > 0, colSums(tab) > 0]
+  if(drop) tab <- tab[rowSums(tab) > 0, colSums(tab) > 0, drop=FALSE]
   dn <- names(dimnames(tab))
-  if (total) tab <- cbind(tab, Ensemble=apply(tab,1,sum))
+  if (total) {
+    .tmp.colnames <- colnames(tab)
+    tab <- cbind(tab, apply(tab,1,sum))
+    colnames(tab) <- c(.tmp.colnames, gettext("All"))
+  }
+  if (n) effectifs <- apply(tab,2,sum)
   tab <- prop.table(tab,2)*100
-  if (total) tab <- rbind(tab,Total=apply(tab,2,sum))
+  if (total) {
+    .tmp.rownames <- rownames(tab)
+    tab <- rbind(tab,Total=apply(tab,2,sum))
+    rownames(tab) <- c(.tmp.rownames, gettext("Total"))
+  }
+  if (n) tab <- rbind(tab, n=effectifs)
   result <- as.table(tab)
   names(dimnames(result)) <- dn
   class(result) <- c("proptab", class(result))
   attr(result, "percent") <- percent
   attr(result, "digits") <- digits
+  attr(result, "total") <- total
+  attr(result, "row.n") <- n
   return(result)
 }
 
@@ -140,6 +177,7 @@ function (tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE) {
 #' @param total if \code{TRUE}, add a column with the sum of percentages and a row with global percentages
 #' @param percent if \code{TRUE}, add a percent sign after the values when printing
 #' @param drop if \code{TRUE}, lines or columns with a sum of zero, which would generate \code{NaN} percentages, are dropped.
+#' @param n if \code{TRUE}, display number of observations per row.
 #' @return
 #' The result is an object of class \code{table} and \code{proptab}.
 #' @seealso
@@ -155,18 +193,30 @@ function (tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE) {
 #' @export rprop lprop
 
 `rprop` <-
-function(tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE) {
+function(tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE, n=FALSE) {
   # subset to non-empty rows/columns
-  if(drop) tab <- tab[rowSums(tab) > 0, colSums(tab) > 0]
+  if(drop) tab <- tab[rowSums(tab) > 0, colSums(tab) > 0, drop=FALSE]
   dn <- names(dimnames(tab))
-  if (total) tab <- rbind(tab, Ensemble=apply(tab,2,sum))
+  if (total) {
+    .tmp.rownames <- rownames(tab)
+    tab <- rbind(tab, apply(tab,2,sum))
+    rownames(tab) <- c(.tmp.rownames, gettext("All"))
+  }
+  if (n) effectifs <- apply(tab,1,sum)
   tab <- prop.table(tab,1)*100
-  if (total) tab <- cbind(tab, Total=apply(tab,1,sum))
+  if (total) {
+    .tmp.colnames <- colnames(tab)
+    tab <- cbind(tab, Total=apply(tab,1,sum))
+    colnames(tab) <- c(.tmp.colnames, gettext("Total"))
+  }
+  if (n) tab <- cbind(tab, n=effectifs)
   result <- as.table(tab)
   names(dimnames(result)) <- dn
   class(result) <- c("proptab", class(result))
   attr(result, "percent") <- percent
   attr(result, "digits") <- digits
+  attr(result, "total") <- total
+  attr(result, "col.n") <- n
   return(result)
 }
 lprop <- rprop
@@ -180,6 +230,7 @@ lprop <- rprop
 #' @param total if \code{TRUE}, add a column with the sum of percentages and a row with global percentages
 #' @param percent if \code{TRUE}, add a percent sign after the values when printing
 #' @param drop if \code{TRUE}, lines or columns with a sum of zero, which would generate \code{NaN} percentages, are dropped.
+#' @param n if \code{TRUE}, display number of observations per row and per column.
 #' @return
 #' The result is an object of class \code{table} and \code{proptab}.
 #' @seealso
@@ -191,24 +242,40 @@ lprop <- rprop
 #' ## Percentages
 #' prop(tab)
 #' ## Percentages with custom display
-#' prop(tab, digits=2, percent=TRUE, total=FALSE)
+#' prop(tab, digits=2, percent=TRUE, total=FALSE, n=TRUE)
 #' @export
 
 `prop` <-
-function (tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE) {
+function (tab, digits = 1, total = TRUE, percent = FALSE, drop = TRUE, n=FALSE) {
   # subset to non-empty rows/columns
-  if(drop) tab <- tab[rowSums(tab) > 0, colSums(tab) > 0]
+  if(drop) tab <- tab[rowSums(tab) > 0, colSums(tab) > 0, drop=FALSE]
   dn <- names(dimnames(tab))
+  if (n) {
+    l.effectifs <- apply(tab,1,sum)
+    r.effectifs <- apply(tab,2,sum)
+  }
   tmp <- tab/sum(tab)*100
   if (total) {
     tmp <- rbind(tmp,Total=apply(tmp,2,sum))
     tmp <- cbind(tmp,Total=apply(tmp,1,sum))
   }
+  if (n) {
+    ntot <- sum(tab)
+    if (total) {
+      l.effectifs <- c(l.effectifs,ntot)
+      r.effectifs <- c(r.effectifs,ntot)
+    }
+    tmp <- cbind(tmp, n=l.effectifs)
+    tmp <- rbind(tmp, n=c(r.effectifs,ntot))
+  }
   result <- as.table(tmp)
   names(dimnames(result)) <- dn
   class(result) <- c("proptab", class(result))
   attr(result, "percent") <- percent
   attr(result, "digits") <- digits
+  attr(result, "total") <- total
+  attr(result, "row.n") <- n
+  attr(result, "col.n") <- n
   return(result)
 }
 
@@ -282,6 +349,12 @@ function (x, digits=NULL, percent=NULL, justify="right", ...) {
   if (!inherits(x, "proptab")) stop("x must be of class 'proptab'")
   if (is.null(digits)) digits <- attr(x, "digits")
   if (is.null(percent)) percent <- attr(x, "percent")
+  total <- attr(x, "total"); if (is.null(total)) total <- FALSE
+  row.n <- attr(x, "row.n"); if (is.null(row.n)) row.n <- FALSE
+  col.n <- attr(x, "col.n"); if (is.null(col.n)) col.n <- FALSE
+  tmp <- format.default(round(x,0), ...)
+  if (row.n) rn <- tmp[nrow(x),]
+  if (col.n) cn <- tmp[,ncol(x)]
   if (percent) {
     fmt <- paste("%.",digits,"f%%",sep="")
     x[,] <- sprintf(x, fmt=fmt)
@@ -289,6 +362,9 @@ function (x, digits=NULL, percent=NULL, justify="right", ...) {
   }
   else
     result <- format.default(round(x,digits), ...)
+  if (row.n) result[nrow(x),] <- rn
+  if (col.n) result[,ncol(x)] <- cn
+  if (total & row.n & col.n) result[nrow(x),ncol(x)] <- ""
   return(result)
 }
 
